<?php
/*
	Plugin Name: DS-Rating
	Plugin URI: http://sneak.co.nz/plugins/ds-rating/
	Description: A plugin that allows you to assign a simple rating for posts.
	Version: 1.2
	License: GPL
	Author: Mike Harding
	Author URI: http://sneak.co.nz/
*/

/*  Copyright 2007 Mike Harding @ http://sneak.co.nz
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

	
load_plugin_textdomain('DSrating');


/* Define default CSS for ratings layout. To be used as default in admin panel, and for resetting */
$ds_rating_css_default = ".rating-container {
	height: 11px;
	width: 61px;
	background: url(".get_settings('siteurl')."/wp-content/plugins/ds-rating-images/black/rating-off.gif) no-repeat;
	overflow: hidden;
	}
.rating {
	padding-top: 11px;
	height: 0px !important; 
	height /**/:11px; /* for IE5/Win only */
	background: url(".get_settings('siteurl')."/wp-content/plugins/ds-rating-images/black/rating-on.gif) no-repeat;
	text-align: left;
	overflow: hidden;
	}
.zero { width: 0px; }
.zero-and-half { width: 6px; }
.one { width: 11px; }
.one-and-half { width: 18px; }
.two { width: 24px; }
.two-and-half { width: 31px; }
.three { width: 36px; }
.three-and-half { width: 43px; }
.four { width: 50px; }
.four-and-half { width: 55px; }
.five { width: 61px; }";

/* Set up default values for options and add them to the database */
$ds_rating_path = rtrim(get_settings('siteurl'), '/')."/wp-content/plugins/ds-rating-images/black/";
$ds_rating_options = array ('css', $ds_rating_css_default, $ds_rating_path, 'gif');
add_option('ds_rating_options', $ds_rating_options, 'Defines options for the DS-Rating plugin', $autoload);

/* Add option panel to admin interface */
function ds_rating() {
	if (function_exists('add_options_page')) {
		add_options_page('DS-Rating Options', 'DS-Rating Options', 7, __FILE__, 'ds_rating_options');
	}
}

/* Create the plugin options page */
function ds_rating_options() {
	global $posts;
	
	/* If options are updated, show message */
	if (isset($_POST['info_update'])) {
	
	$ds_rating_pathupdate = get_settings('siteurl')."/".$_POST['txtRatingImagePath'];
	/* Add trailing '/' to path if one doesn't already exist */
	if(substr($ds_rating_pathupdate, -1, 1) != '/'){
		$ds_rating_pathupdate = $ds_rating_pathupdate.'/';
	}
	
	$ds_rating_update = array ($_POST['dsRatingOutputType'], $_POST['txtRatingCss'], $ds_rating_pathupdate, $_POST['txtRatingImageType']);
	update_option('ds_rating_options', $ds_rating_update);
	?>
		<div id="message" class="updated">
			<p>DS Rating options have been updated</p>
		</div>	
	<?php } ?>
	
	
	<?php $ds_rating_options = get_option('ds_rating_options');?>
	<div class="wrap">
		<form method="post" action="" id="ds-rating-form">
			<h2>DS-Rating options</h2>
			
			<p>
				When displaying the ratings for a post, there are two options for the type of HTML output: 
				One uses CSS to define the images (this is set as the default), and the other just points straight to image files (one image per rating value). 
			</p>
			
			<fieldset id="ds-rating-output">
				<legend><?php _e('Display options', 'Localization name') ?></legend>
				<div>
					<label><input type="radio" name="dsRatingOutputType" id="ds-rating-output-css" value="css"<?php if($ds_rating_options[0] == 'css'){ echo ' checked'; } ?>> CSS</label>
					<label><input type="radio" name="dsRatingOutputType" id="ds-rating-output-image" value="images"<?php if($ds_rating_options[0] == 'images'){ echo ' checked'; } ?>> Image files</label>
				</div>
			</fieldset>
				
			<fieldset id="ds-rating-css">
				<legend><?php _e('CSS for the rating display (if using CSS option)', 'Localization name') ?></legend>
				<div>
					<textarea name="txtRatingCss" rows="15" cols="70"><?php echo $ds_rating_options[1]; ?></textarea>
				</div>
			</fieldset>
			
			<?php				
				$ds_siteurl = get_settings('siteurl');
				$ds_imagepath = $ds_rating_options[2];
				$ds_relative_imagepath = substr($ds_imagepath, strlen($ds_siteurl)+1, strlen($ds_imagepath));
			?>
			<fieldset id="ds-rating-imagepath">
				<legend><?php _e('Path to image files (if using image option)', 'Localization name') ?></legend>
				<div>
					<label for="txtRatingImagePath"><?php echo $ds_siteurl."/"; ?> </label>
					<input type="text" name="txtRatingImagePath" size="45" value="<?php echo $ds_relative_imagepath; ?>">
					<label for="txtRatingImageType" class="image-type"><?php _e('Image type: ', 'Localization name') ?></label>
					<select name="txtRatingImageType">
						<option value="gif"<?php if($ds_rating_options[3] == 'gif'){ echo ' selected'; } ?>>GIF</option>
						<option value="jpg"<?php if($ds_rating_options[3] == 'jpg'){ echo ' selected'; } ?>>JPG</option>
						<option value="png"<?php if($ds_rating_options[3] == 'png'){ echo ' selected'; } ?>>PNG</option>
					</select>
				</div>
			</fieldset>
				
			<div class="submit">
				<input type="submit" name="info_update" value="<?php _e('Update options', 'Localization name') ?> &raquo;" />
			</div>
				
			
		</form>
	</div><?php
}


/* Function to check if a post has a rating assigned to it.
   
   By default it will use the ID of the current post when looking up the rating, however if you
   are calling the function outside the main article then you can provide a post ID as a parameter.

*/
function has_rating($single_post_id = ''){
	global $post;
	if ($single_post_id != ''):
		$this_post_id = $single_post_id;
	else:
		$this_post_id = $post->ID;
	endif;
	
	$has_rating = get_post_meta($this_post_id, '_ds_rating', true);
	if ($has_rating != null){
		$has_rating = (bool) true;
	} else {
		$has_rating = (bool) false;
	}
	
	if ($has_rating):
		return true;
	else:
		return false;
	endif;
}

/* Function to return the rating value for a post */
function get_rating($id = ''){
	global $wpdb, $post;
	if ($id == ''){
		$id = $post->ID;
	}
	if ($posts = $wpdb->get_results("SELECT meta_value FROM $wpdb->postmeta WHERE meta_key = '_ds_rating' AND post_id = ".$id." LIMIT 1")) {
		foreach ($posts as $rating) {
			$get_rating = $rating->meta_value;
		}
		
		switch ($get_rating){
		/* Convert numeric values to text ones - CSS does not allow periods in class names */
		case "0":
			$rating_class = 'zero';
			break;
		case "0.5":
			$rating_class = 'zero-and-half';
			break;
		case "1":
			$rating_class = 'one';
			break;
		case "1.5":
			$rating_class = 'one-and-half';
			break;
		case "2":
			$rating_class = 'two';
			break;
		case "2.5":
			$rating_class = 'two-and-half';
			break;
		case "3":
			$rating_class = 'three';
			break;
		case "3.5":
			$rating_class = 'three-and-half';
			break;
		case "4":
			$rating_class = 'four';
			break;
		case "4.5":
			$rating_class = 'four-and-half';
			break;
		case "5":
			$rating_class = 'five';
			break;
		}
		
		/* Output the relevant XHTML */
		$ds_rating_options = get_option('ds_rating_options');
		if ($ds_rating_options[0] == 'css') {
			/* Output the rating code block, for using CSS to display */
			echo '<div class="rating-container"><div class="rating '.$rating_class.'">Rating: '.$get_rating.'</div></div>';
		} else if($ds_rating_options[0] == 'images') {	
			/* Output an image */
			echo '<img src="'.$ds_rating_options[2].str_replace('.','-',$get_rating).'.gif" alt="This entry has a rating of '.$get_rating.'" class="rating-image" />';
		}
	}
}

/* Output a list of rated posts */

function list_rated_posts($prefix,$suffix,$limit,$orderby,$summary){
	global $wpdb, $post;
	
	if ($prefix == ''){ $prefix = '<li>'; }
	if ($suffix == ''){ $suffix = '</li>'; }
	if ($limit == ''){ $limit = '5'; }
	if ($orderby == ''){ $orderby = 'post_date'; }
	if (!isset($summary)){ $summary = TRUE; }
	
	if ($orderby == 'rated_high'){
		$orderby = 'meta_value';
		$ordertype = 'DESC';
	} else if ($orderby == 'rated_low'){
		$orderby = 'meta_value';
		$ordertype = 'ASC';
	} else {
		$ordertype = 'DESC';
	}
	
	$rating_query = "SELECT * FROM $wpdb->posts LEFT JOIN $wpdb->postmeta ON ($wpdb->posts.ID = $wpdb->postmeta.post_id) ";
	$rating_query .= "WHERE meta_key = '_ds_rating' ORDER BY ".$orderby." ".$ordertype.", post_date DESC";
	
	$list_count = 0;
	if ($posts = $wpdb->get_results($rating_query)) {
		foreach ($posts as $rated_post) {	
			if($rated_post->meta_value != null) {
				
				echo $prefix;
			
				$item_title = wptexturize($rated_post->post_title);
				$item_link = '<a href="'.get_permalink($rated_post->ID).'" title="'.$item_title.'">'.$item_title.'</a>';

				echo '<h4>'.$item_link.'</h4>';
				if ($summary){
					if ($rated_post->post_excerpt){
						echo wpautop(wptexturize($rated_post->post_excerpt));
					}
				}
				get_rating($rated_post->ID);

				echo $suffix;				
				$list_count++;
				
				if($list_count == $limit){
					break;
				}
			}
		}
	}
}

/* Set up the rating entry for the admin area */

function ds_rating_input() {
	global $post;
	
	// get the current rating
	$current_rating = get_post_meta($post->ID,'_ds_rating',true);
	
	switch ($current_rating){
		case "0":
			$selected0 = 'checked="checked" ';
			break;
		case "0.5":
			$selected05 = 'checked="checked" ';
			break;
		case "1":
			$selected1 = 'checked="checked" ';
			break;
		case "1.5":
			$selected15 = 'checked="checked" ';
			break;
		case "2":
			$selected2 = 'checked="checked" ';
			break;
		case "2.5":
			$selected25 = 'checked="checked" ';
			break;
		case "3":
			$selected3 = 'checked="checked" ';
			break;
		case "3.5":
			$selected35 = 'checked="checked" ';
			break;
		case "4":
			$selected4 = 'checked="checked" ';
			break;
		case "4.5":
			$selected45 = 'checked="checked" ';
			break;
		case "5":
			$selected5 = 'checked="checked" ';
			break;
		default:
			$selectedNull = 'checked="checked" ';
	}

	// print the rating fields into the Write Post form
	
	echo '<fieldset id="ds-rating-post">';
	echo '<legend><a href="http://sneak.co.nz/plugins/ds-rating/" title="Find out more about DS-Rating">' . __('Rating', 'DSrating') . '</a></legend>';
	?>
	<input type="hidden" name="ds-rating-verification" id="ds-rating-verification" value="<?php if(function_exists('wp_create_nonce')){ echo wp_create_nonce('ds-rating'); } ?>" />
	<?php
	echo '<ul id="ds-rating-inputs">';
	echo '	<li class="no-rating"><input type="radio" name="rdo_ds_rating" id="dsrNull" value="" '. $selectedNull . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr0" value="0" '. $selected0 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr05" value="0.5" '. $selected05 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr1" value="1" '. $selected1 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr15" value="1.5" '. $selected15 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr2" value="2" '. $selected2 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr25" value="2.5" '. $selected25 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr3" value="3" '. $selected3 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr35" value="3.5" '. $selected35 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr4" value="4" '. $selected4 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr45" value="4.5" '. $selected45 . '/></li>';
	echo '	<li><input type="radio" name="rdo_ds_rating" id="dsr5" value="5" '. $selected5 . '/></li>';
	echo '</ul>';
	echo '<ul id="ds-rating-labels">';
	echo '	<li class="no-rating"><label for="dsrNull">'. __('No rating', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr0">'. __('0', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr05">'. __(' ', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr1">'. __('1', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr15">'. __(' ', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr2">'. __('2', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr25">'. __(' ', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr3">'. __('3', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr35">'. __(' ', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr4">'. __('4', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr45">'. __(' ', 'DSrating') . '</label></li>';
	echo '	<li><label for="dsr5">'. __('5', 'DSrating') . '</label></li>';
	echo '</ul>';
	echo '</fieldset>';
}

function ds_rating_update_post($id) {
	// authorization
	if (current_user_can('edit_post', $id)){
		// verify origination of update
		if (function_exists('wp_verify_nonce')){
			if (wp_verify_nonce($_POST['ds-rating-verification'], 'ds-rating')){
				delete_post_meta($id, '_ds_rating');
				$setting = (isset($_POST["rdo_ds_rating"])) ? $_POST["rdo_ds_rating"] : null;
				add_post_meta($id, '_ds_rating', $setting);
			}
		} else {
			delete_post_meta($id, '_ds_rating');
			$setting = (isset($_POST["rdo_ds_rating"])) ? $_POST["rdo_ds_rating"] : null;
			add_post_meta($id, '_ds_rating', $setting);
		}
	}
}

function ds_rating_styles($id) {	
	$ds_rating_options = get_option('ds_rating_options');
	echo '
<style type="text/css" media="all"><!--
	';
	echo $ds_rating_options[1];
	echo '
--></style>
	';
}

function ds_rating_admin_styles($id) {	
	echo '<style type="text/css" media="all"><!--';
	echo '	fieldset#ds-rating-post { border: 1px solid #ccc; padding: 0.5em; margin-bottom: 1em; }';
	echo '	fieldset#ds-rating-post ul { clear: left; margin: 0; padding: 0; height: 1.5em; }';
	echo '	fieldset#ds-rating-post ul li { display: inline; float: left; text-align: center; width: 2.5em; color: #666; font-size: 0.8em; }';
	echo '	fieldset#ds-rating-post ul li.no-rating { width: 6.5em; }';
	echo '	form#ds-rating-form fieldset { border: 1px solid #ccc; padding: 1em; margin-bottom: 1em; }';
	echo '	form#ds-rating-form fieldset legend { color: #666; }';
	echo '	form#ds-rating-form fieldset#ds-rating-output label { margin-right: 1.5em; }';
	echo '	form#ds-rating-form .ds-note { color: #999; }';
	echo '	form#ds-rating-form fieldset label.image-type { margin-left: 1em; }';
	echo '--></style>';
}

add_action('wp_head', 'ds_rating_styles');
add_action('admin_head', 'ds_rating_admin_styles');
add_action('edit_form_advanced', 'ds_rating_input');
add_action('simple_edit_form', 'ds_rating_input');
add_action('admin_menu', 'ds_rating');
add_action('save_post', 'ds_rating_update_post');
add_action('edit_post', 'ds_rating_update_post');
add_action('publish_post', 'ds_rating_update_post');

?>
